#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2009 Andreas Böttger 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""
gudev enables you ...
"""

from optparse import OptionParser
import gettext
import os
import os.path
import platform
import sys


try:
    import dbus
    import dbus.glib
    have_dbus = True
except ImportError:
    print >> sys.stderr, """
    Warning: python-dbus not found. Disabling D-Bus support.
    """
    have_dbus = False


if __name__ == '__main__':
    # Paths to important files
    gudev_script = sys.argv[0]
    if os.path.islink(gudev_script):
        gudev_script = os.readlink(gudev_script)
    gudev_dir = os.path.join(os.path.dirname(gudev_script), '..')
    prefix = os.path.abspath(os.path.normpath(gudev_dir))

    src_dir = os.path.join(prefix, 'src')
    data_dir = os.path.join(prefix, 'data')
    locale_dir = os.path.join(prefix, 'share', 'locale')
    ui_folder = os.path.join(prefix, 'share', 'gudev', 'ui')
    credits_file = os.path.join(prefix, 'share', 'gudev', 'credits.txt')
    icon_file = os.path.join(prefix, 'share', 'icons', 'hicolor', 'scalable', 'apps', 'gudev.svg')

    if os.path.exists(src_dir) and os.path.exists(data_dir) and \
            not prefix.startswith('/usr'):
        # Run gUdev from local source folder (not installed)
        print >> sys.stderr, 'Using modules from', src_dir
        sys.path.insert(0, src_dir)
        print >> sys.stderr, 'Using resources from', data_dir
        locale_dir = os.path.join(data_dir, 'locale')
        ui_folder = os.path.join(data_dir, 'ui')
        credits_file = os.path.join(data_dir, 'credits.txt')
        icon_file = os.path.join(data_dir, 'gudev.svg')

    # Set up the path to translation files
    gettext.bindtextdomain('gudev', locale_dir)

    import gudev

    # Enable i18n for gUdev translations
    _ = gudev.gettext

    # Set up paths to folder with GtkBuilder files and gudev.svg
    gudev.ui_folders.append(ui_folder)
    gudev.credits_file = credits_file
    gudev.icon_file = icon_file

    s_usage = 'usage: %%prog [options]\n\n%s' % (__doc__.strip())
    s_version = '%%prog %s' % (gudev.__version__)

    parser = OptionParser(usage=s_usage, version=s_version)

    parser.add_option("-v", "--verbose",
                      action="store_true", dest="verbose", default=False,
                      help=_("Print debugging output to stdout"))

    (options, args) = parser.parse_args(sys.argv)

    gudev.ui_folders.insert(0, os.path.join(ui_folder, 'desktop'))

    if options.verbose:
        from gudev.liblogger import enable_verbose
        enable_verbose()

    if have_dbus:
        # Try to find an already-running instance of gUdev
        try:
            session_bus = dbus.SessionBus()
            remote_object = session_bus.get_object(gudev.dbus_bus_name, gudev.dbus_gui_object_path)
            from gudev.liblogger import log
            log('Found gUdev GUI instance already running')
        except dbus.exceptions.DBusException:
            remote_object = None
    else:
        # No D-Bus available :/
        remote_object = None

    if remote_object is not None:
        # An instance of GUI is already running
        remote_object.show_gui_window(dbus_interface=gudev.dbus_interface)
    else:
        # gUdev is not yet running - create new instance
        from gudev import gui

        # check if we have an X11 connection (but not on Windows)
        if not os.environ.get('DISPLAY', None):
            print >> sys.stderr, 'Cannot start gUdev: $DISPLAY is not set.'
            sys.exit(1)

        gui.main(options)

