#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2009 Andreas Böttger 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from distutils.core import setup
import glob
import gudev
import os
import re
import sys

# build targets
(DEFAULT) = range(1)

# import the gudev module locally for module metadata
sys.path.insert(0, 'src')

# if we are running "setup.py sdist", include all targets (see below)
building_source = ('sdist' in sys.argv)

# build target
target = DEFAULT

# search for translations and repare to install
translation_files = []
for mofile in glob.glob('data/locale/*/LC_MESSAGES/gudev.mo'):
    modir = os.path.dirname(mofile).replace('data', 'share')
    translation_files.append((modir, [mofile]))

if not len(translation_files) and not 'clean' in sys.argv:
    print >> sys.stderr, """
    Warning: No translation files. (Did you forget to run "make messages"?)
    """

# files to install
inst_manpages = glob.glob('doc/man/*.1')
inst_share_ui = glob.glob('data/ui/*.ui')
inst_share_ui_desktop = glob.glob('data/ui/desktop/*.ui')

inst_share_gudev = [ 'data/credits.txt' ]
inst_desktop = [ 'data/gudev.desktop' ]

inst_icons = [ 'data/gudev.png' ]
inst_icons_64 = [ 'data/icons/64/gudev.png' ]
inst_icons_40 = [ 'data/icons/40/gudev.png' ]
inst_icons_26 = [ 'data/icons/26/gudev.png' ]
inst_icons_24 = [ 'data/icons/24/gudev.png' ]
inst_icons_22 = [ 'data/icons/22/gudev.png' ]
inst_icons_16 = [ 'data/icons/16/gudev.png' ]
inst_icons_svg = [ 'data/gudev.svg' ]

data_files = [
  ('share/man/man1', inst_manpages),
  ('share/gudev/ui', inst_share_ui),
  ('share/pixmaps', inst_icons),
  ('share/gudev', inst_share_gudev),
]

packages = [
  'gudev',
  'gudev.gtkui',
  'gudev.gtkui.interface',
]

# target-specific installation data files
if target == DEFAULT or building_source:
    data_files += [
      ('share/gudev/ui/desktop', inst_share_ui_desktop),
      ('share/applications', inst_desktop),
      ('share/icons/hicolor/scalable/apps', inst_icons_svg),
      ('share/icons/hicolor/48x48/apps', inst_icons),
      ('share/icons/hicolor/24x24/apps', inst_icons_24),
      ('share/icons/hicolor/22x22/apps', inst_icons_22),
      ('share/icons/hicolor/16x16/apps', inst_icons_16),
    ]
    packages += [
      'gudev.gtkui.desktop',
    ]
    additional_scripts = []

if target == building_source:
    data_files += [
      ('share/icons/hicolor/scalable/apps', inst_icons_64),
      ('share/icons/hicolor/40x40/apps', inst_icons_40),
      ('share/icons/hicolor/26x26/apps', inst_icons_26),
    ]
    packages += []
    additional_scripts = []

author, email = re.match(r'^(.*) <(.*)>$', gudev.__author__).groups()

setup(
  name='gudev',
  version=gudev.__version__,
  package_dir={ '':'src' },
  packages=packages,
  description='udev gui',
  author=author,
  author_email=email,
  url=gudev.__url__,
  scripts=glob.glob('bin/*') + additional_scripts,
  data_files=data_files + translation_files
)

