# -*- coding: utf-8 -*-
# Copyright (C) 2009 Andreas Böttger 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#
# inspired by: 
# gPodder - A media aggregator and podcast client
# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
#


from gudev.gtkui.interface.common import BuilderWidget
from gudev.gtkui.interface.configeditor import gUdevConfigEditor
import gtk
import gudev


_ = gudev.gettext


class gUdevPreferences(BuilderWidget):
    def new(self):
        if not hasattr(self, 'callback_finished'):
            self.callback_finished = None
                    
        self._config.connect_gtk_editable( 'editor', self.openApp)
                
        # setup cell renderers
        cellrenderer = gtk.CellRendererPixbuf()
        self.comboEditorApp.pack_start(cellrenderer, False)
        self.comboEditorApp.add_attribute(cellrenderer, 'pixbuf', 2)
        cellrenderer = gtk.CellRendererText()
        self.comboEditorApp.pack_start(cellrenderer, True)
        self.comboEditorApp.add_attribute(cellrenderer, 'markup', 0)
        
        if not hasattr(self, 'user_apps_reader'):
            self.user_apps_reader = UserAppsReader(['text/plain'])

        self.comboEditorApp.set_row_separator_func(self.is_row_separator)

        self.user_apps_reader.read()

        self.comboEditorApp.set_model(self.user_apps_reader.get_applications_as_model('text/plain'))
        index = self.find_active_editor_app()
        self.comboEditorApp.set_active(index)

    def find_active_editor_app(self):
        index_custom = -1
        model = self.comboEditorApp.get_model()
        iter = model.get_iter_first()
        index = 0
        while iter is not None:
            command = model.get_value(iter, 1)
            if command == self.openApp.get_text():
                return index
            if index_custom < 0 and command == '':
                index_custom = index
            iter = model.iter_next(iter)
            index += 1
        # return index of custom command or first item
        return max(0, index_custom)
            
    def is_row_separator(self, model, iter):
        return model.get_value(iter, 0) == ''

    def on_gUdevPreferences_destroy(self, widget, *args):
        self.on_btnOK_clicked(widget, *args)

    def on_btnConfigEditor_clicked(self, widget, *args):
        gUdevConfigEditor(self.gUdevPreferences, _config=self._config)
        self.on_btnOK_clicked(widget, *args)

    def on_cbEnvironmentVariables_toggled(self, widget, *args):
         sens = not self.cbEnvironmentVariables.get_active()
         self.httpProxy.set_sensitive(sens)

    def on_btnOK_clicked(self, widget, *args):
        self.gUdevPreferences.destroy()
        if self.callback_finished:
            self.callback_finished()
            
    def on_comboEditorApp_changed(self, widget, *args):
        # find out which one
        iter = self.comboEditorApp.get_active_iter()
        model = self.comboEditorApp.get_model()
        command = model.get_value( iter, 1)
        if command == '':
            if self.openApp.get_text() == 'default':
                self.openApp.set_text('')
            self.openApp.set_sensitive( True)
            self.openApp.show()
            self.labelCustomCommand.show()
        else:
            self.openApp.set_text( command)
            self.openApp.set_sensitive( False)
            self.openApp.hide()
            self.labelCustomCommand.hide()
