# -*- coding: utf-8 -*-
# Copyright (C) 2009 Andreas Böttger 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#
# inspired by: 
# gPodder - A media aggregator and podcast client
# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
#


import gtk
import os
import shutil
import sys

import gudev

_ = gudev.gettext

from gudev import util

from gudev.gtkui.base import GtkBuilderWidget

from gudev.gtkui.widgets import NotificationWindow

try:
    import pynotify
    if pynotify.init('gUdev'):
        try:
            pynotify_server_info = pynotify.get_server_info()
            pynotify_server = pynotify_server_info.get('name', 'unknown')
            notify_server_from_canonical = (pynotify_server == 'notify-osd')
            if notify_server_from_canonical:
                print >> sys.stderr, 'Broken libnotify server found.'
                print >> sys.stderr, '(notify-osd does not positioning)'
        except:
            print >> sys.stderr, 'Error detecting libnotify server.'
            pynotify = None
            notify_server_from_canonical = False
    else:
        pynotify = None
        notify_server_from_canonical = False
except ImportError:
    pynotify = None
    notify_server_from_canonical = False

class BuilderWidget(GtkBuilderWidget):

    def __init__(self, parent, **kwargs):
        self._window_iconified = False
        GtkBuilderWidget.__init__(self, gudev.ui_folders, gudev.textdomain, **kwargs)

        # Enable support for tracking iconified state
        if hasattr(self, 'on_iconify') and hasattr(self, 'on_uniconify'):
            self.main_window.connect('window-state-event', \
                    self._on_window_state_event_iconified)

        if parent is not None:
            self.main_window.set_transient_for(parent)

            if hasattr(self, 'center_on_widget'):
                (x, y) = parent.get_position()
                a = self.center_on_widget.allocation
                (x, y) = (x + a.x, y + a.y)
                (w, h) = (a.width, a.height)
                (pw, ph) = self.main_window.get_size()
                self.main_window.move(x + w / 2 - pw / 2, y + h / 2 - ph / 2)

    def _on_window_state_event_iconified(self, widget, event):
        if event.new_window_state & gtk.gdk.WINDOW_STATE_ICONIFIED:
            if not self._window_iconified:
                self._window_iconified = True
                self.on_iconify()
        else:
            if self._window_iconified:
                self._window_iconified = False
                self.on_uniconify()

        return False

    def is_iconified(self):
        return self._window_iconified

    def notification(self, message, title=None, important=False, widget=None):
        util.idle_add(self.show_message, message, title, important, widget)

class TreeViewHelper(object):
    """Container for gUdev-specific TreeView attributes."""
    BUTTON_PRESS = '_gudev_button_press'
    LAST_TOOLTIP = '_gudev_last_tooltip'
    CAN_TOOLTIP = '_gudev_can_tooltip'
    ROLE = '_gudev_role'

    # Enum for the role attribute
    ROLE_PODCASTS, ROLE_EPISODES, ROLE_DOWNLOADS = range(3)

    @classmethod
    def set(cls, treeview, role):
        setattr(treeview, cls.BUTTON_PRESS, (0, 0))
        setattr(treeview, cls.LAST_TOOLTIP, None)
        setattr(treeview, cls.CAN_TOOLTIP, True)
        setattr(treeview, cls.ROLE, role)

    @classmethod
    def save_button_press_event(cls, treeview, event):
        setattr(treeview, cls.BUTTON_PRESS, (event.x, event.y))
        return True

    @classmethod
    def get_button_press_event(cls, treeview):
        return getattr(treeview, cls.BUTTON_PRESS)

    @staticmethod
    def make_search_equal_func(gudev_model):
        def func(model, column, key, iter):
            if model is None:
                return True
            key = key.lower()
            for column in gudev_model.SEARCH_COLUMNS:
                if key in model.get_value(iter, column).lower():
                    return False
            return True
        return func

