# -*- coding: utf-8 -*-
# Copyright (C) 2009 Andreas Böttger 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#
# inspired by: 
# gPodder - A media aggregator and podcast client
# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
#


from gudev import util
from gudev.gtkui.config import UIConfig
from gudev.gtkui.desktop.preferences import gUdevPreferences
from gudev.gtkui.desktopfile import UserAppsReader
from gudev.gtkui.draw import draw_text_box_centered
from gudev.gtkui.interface.common import BuilderWidget, TreeViewHelper
from gudev.gtkui.widgets import SimpleMessageArea
from gudev.liblogger import log
from pyinotify import WatchManager, EventsCodes, ThreadedNotifier, ProcessEvent
from pywatch.watcher import Watcher
from xml.sax import saxutils
import collections
import fnmatch
import glob
import gobject
import gtk
import gtk.gdk
import gudev
import os
import pango
import shutil
import subprocess
import sys
import tempfile
import threading
import time
import urllib
import urllib2


try:
    import dbus
    import dbus.service
    import dbus.mainloop
    import dbus.glib
except ImportError:
    # Mock the required D-Bus interfaces with no-ops (ugly? maybe.)
    class dbus:
        class SessionBus:
            def __init__(self, *args, **kwargs):
                pass
        class glib:
            class DBusGMainLoop:
                pass
        class service:
            @staticmethod
            def method(interface):
                return lambda x: x
            class BusName:
                def __init__(self, *args, **kwargs):
                    pass
            class Object:
                def __init__(self, *args, **kwargs):
                    pass



_ = gudev.gettext

class pTmp(ProcessEvent):
    
    def process_IN_MODIFY(self, event):
        log("Modified: %s" % os.path.join(event.path, event.name))

    def process_IN_IGNORED(self, event):
        log("Removed from watch list: %s" % os.path.join(event.path, event.name))            

class gUdev(BuilderWidget, dbus.service.Object):
    
    devices = dict()
    
    def __init__(self, bus_name, config):
        dbus.service.Object.__init__(self, object_path=gudev.dbus_gui_object_path, bus_name=bus_name)
        self.config = config
        BuilderWidget.__init__(self, None)
        
        self.watcher = None
        
#        self.watch_manager = WatchManager()
#        self.watched = None
#        self.mask = EventsCodes.IN_MODIFY
#        notifier = ThreadedNotifier(self.watch_manager, pTmp())
#        notifier.start()

    def new(self):
        
        self.toolbar.set_property('visible', self.config.show_toolbar)
        
        self.config.connect_gtk_window(self.gUdev, 'main_window')
        self.main_window.show()

#        self.gUdev.connect('key-press-event', self.on_key_press)

        self.config.add_observer(self.on_config_changed)

        self.default_title = None
        title = self.gUdev.get_title()
        if title is not None:
            self.set_title(title)
        else:
            self.set_title(_('gudev'))
            
        self.set_devices()
        self.set_rules()

        self.devicesAvailable.set_active(0)
        
        self.currently_updating = False

        self.context_menu_mouse_button = 3

        # Message area
        self.message_area = None
        
        self.filename = None
        
        # load list of user applications for editing
        self.user_apps_reader = UserAppsReader(['text/plain'])
        def read_apps():
            time.sleep(3) # give other parts of gUdev a chance to start up
            self.user_apps_reader.read()
            util.idle_add(self.user_apps_reader.get_applications_as_model, 'text/plain', False)
        threading.Thread(target=read_apps).start()
        

        
    def watch_file(self):
        if self.watcher is not None:
            self.watcher.stop_monitor()
            
        self.watcher = Watcher([self.filename], \
                  callback=self.change_ruleFile, \
                  verbose=True)
        self.watcher.monitor()        
        
    def set_title(self, new_title):
        self.default_title = new_title
        self.gUdev.set_title(new_title)
        
    def set_rules(self):
        udev_rules_path = self.config.udev_rules_path

        model = self.ruleFiles.get_model()
        model.clear()
                
        for path, dirs, files in os.walk(udev_rules_path):
            """ only rule files
            """
            for file in [os.path.abspath(os.path.join(path, filename)) for filename in files if fnmatch.fnmatch(filename, '*.rules')]:
                model.append([file])

        
    def set_devices(self):
        usb_storage_path = self.config.usb_storage_path
        
        if os.path.isdir(usb_storage_path):
            model = self.devicesAvailable.get_model()
            model.clear()

            for root, dirs, files in os.walk(usb_storage_path):
                for file in files:
                    data = (str(open(usb_storage_path + file, "r").read()))
                    
                    hostIndex = data.find("Host scsi") + len("Host scsi0: ")
                    vendorIndex = data.find("Vendor: ") + len("Vendor: ")
                    productIndex = data.find("Product: ") + len("Product: ")
                    serialIndex = data.find("Serial Number: ") + len("Serial Number: ")
                    protocolIndex = data.find("Protocol: ") + len("Protocol: ")
                    
                    host = data[hostIndex:vendorIndex - len("Vendor: ")].strip().replace("""\n""", "")
                    vendor = data[vendorIndex:productIndex - len("Product: ")].strip().replace("""\n""", "")
                    product = data[productIndex:serialIndex - len("Serial Number: ")].strip().replace("""\n""", "")
                    serial = data[serialIndex:protocolIndex - len("Protocol: ")].strip().replace("""\n""", "")
                    
                    gUdev.devices[serial] = dict({"host": host, "vendor": vendor, \
                                        "product": product})

                    model.append([product + " (" + vendor + ") ", serial, host, vendor, product])
                    
    def edit_udev_rules_for_real(self, filename):
        if self.config.editor and self.config.editor != 'default':
            for command in util.format_desktop_command(self.config.editor, [filename]):
                log('Executing: %s', repr(command), sender=self)
                subprocess.Popen(command)
        else:
            log('Opening with system default: %s', filename, sender=self)
            util.gui_open(filename)



                
    def get_selected_udev_rules(self):
        """Get the selected udev rule file"""
        model = self.ruleFiles.get_model()
        iter = self.ruleFiles.get_active_iter()
        
        if iter > -1:
            return model.get_value(iter, 0)

    def edit_udev_rules(self, file):
        try:
            self.edit_udev_rules_for_real(file)
        except Exception, e:
            log('Error in opening!', sender=self, traceback=True)
            self.show_message(_('Please check your editor settings in the preferences dialog.'), _('Error opening editor'), widget=self.toolPreferences)
            
    def on_btn_edit_clicked(self, widget):
        self.edit_udev_rules(self.get_selected_udev_rules())

    def on_refresh_devices_clicked(self, widget=None):
        self.set_devices()
        
    def on_ruleFiles_changed(self, widget=None):
        model_ruleFiles = self.ruleFiles.get_model()
        iter_ruleFiles = self.ruleFiles.get_active_iter()

        self.filename = model_ruleFiles.get_value(iter_ruleFiles, 0)
        self.change_ruleFile() 
        self.watch_file()      
              
#        # Watch file modifications
#        if self.watched is not None:
#            self.watch_manager.rm_watch(self.watched.values())
#            
#        self.watched = self.watch_manager.add_watch(model_ruleFiles.get_value(iter_ruleFiles, 0), self.mask)

    def change_ruleFile(self):
        buffer_rulesAvailable = self.rulesAvailable.get_buffer()
        
        rules = open(self.filename, "r")

        buffer_rulesAvailable.delete(buffer_rulesAvailable.get_start_iter(), \
                                     buffer_rulesAvailable.get_end_iter())
        for rule in rules:        
            buffer_rulesAvailable.insert(buffer_rulesAvailable.get_end_iter(), rule)
        
    def on_devicesAvailable_changed(self, widget=None):
        model = self.devicesAvailable.get_model()
        iter = self.devicesAvailable.get_active_iter()
        
        if iter > -1:
            self.entry_serial_number.set_text(model.get_value(iter, 1))
            self.entry_host.set_text(model.get_value(iter, 2))
            self.entry_vendor.set_text(model.get_value(iter, 3))
            self.entry_product.set_text(model.get_value(iter, 4))

    def on_config_changed(self, name, old_value, new_value):
        if name == 'show_toolbar':
            self.toolbar.set_property('visible', new_value)

    def on_gUdev_delete_event(self, widget, *args):
        self.close_gudev()
        return True

    def close_gudev(self):
        """ clean everything and exit properly
        """

        self.gUdev.hide()

        while gtk.events_pending():
            gtk.main_iteration(False)

        self.quit()
        sys.exit(0)
        
    def properties_closed(self):
        NotImplemented

    def on_itemPreferences_activate(self, widget, *args):
        gUdevPreferences(self.gUdev, _config=self.config, \
                callback_finished=self.properties_closed, \
                user_apps_reader=self.user_apps_reader)
 
    @dbus.service.method(gudev.dbus_interface)
    def show_gui_window(self):
        self.gUdev.present()

def main(options=None):
    gobject.threads_init()
    gobject.set_application_name('gudev')

    gtk.window_set_default_icon_name('gudev')
    gtk.about_dialog_set_url_hook(lambda dlg, link, data: util.open_website(link), None)

    try:
        session_bus = dbus.SessionBus(mainloop=dbus.glib.DBusGMainLoop())
        bus_name = dbus.service.BusName(gudev.dbus_bus_name, bus=session_bus)
    except dbus.exceptions.DBusException, dbe:
        log('Warning: Cannot get "on the bus".', traceback=True)
        dlg = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, \
                gtk.BUTTONS_CLOSE, _('Cannot start gudev'))
        dlg.format_secondary_markup(_('D-Bus error: %s') % (str(dbe),))
        dlg.set_title('gudev')
        dlg.run()
        dlg.destroy()
        sys.exit(0)

    util.make_directory(gudev.home)
    config = UIConfig(gudev.config_file)
    
    gp = gUdev(bus_name, config)

    # Handle options
    gp.run()
    

    




